import javax.sound.sampled.*;

/**
   Utility class to read audio in real-time.
   Copyright The University of British Columbia, Canada
   @author Kees van den Doel (kvdoel@cs.ubc.ca)
*/
public class RTAudioIn {
    private float srate;
    private int bitsPerFrame;
    private int nchannels;
    private boolean signed;
    private static final boolean bigEndian = false;
    private AudioFormat audioFormat; 
    private TargetDataLine targetDataLine;
    private int scratchBufsz; // byte buffer size
    private byte[] bbuf;
    private int bufferSize; // JavaSound buffersize
    
    private void initAudio(int bufferSize, float srate,int bitsPerFrame,int nchannels,boolean signed) {
        this.srate = srate;
        this.bufferSize = bufferSize;
        this.bitsPerFrame = bitsPerFrame;
        this.nchannels = nchannels;
        this.signed = signed;
        audioFormat = new AudioFormat(srate,bitsPerFrame,nchannels,signed,bigEndian);
        DataLine.Info info = new DataLine.Info(TargetDataLine.class, audioFormat);
        if (!AudioSystem.isLineSupported(info)) {
            System.out.println(getClass().getName()+" : Error gettting targetDataLine: not supported\n");
        }       
        try {
            targetDataLine = (TargetDataLine) AudioSystem.getLine(info);
            if(bufferSize == 0) {
                targetDataLine.open(audioFormat);
            } else {
                targetDataLine.open(audioFormat,bufferSize);
            }
            targetDataLine.start();
        } catch (LineUnavailableException ex) {
            System.out.println(getClass().getName()+" : Error getting line\n");
        }
        scratchBufsz = 4*2*targetDataLine.getBufferSize();
        bbuf = new byte[scratchBufsz];
    }

    /** Constructor.
        @param bufferSize Buffer size used by JavaSound.
        @param srate sampling rate in Hertz.
        @param bitsPerFrame number of bits per audio frame.
        @param nchannels number of audio channels.
        @param signed true if signed format, false otherwise.
     */
    public RTAudioIn(int buffersize, float srate,int bitsPerFrame,int nchannels,boolean signed) {
        initAudio(buffersize,srate,bitsPerFrame,nchannels,signed);
    }

    /** Constructor. Uses default high latency JavaSound buffer size.
        @param srate sampling rate in Hertz.
        @param bitsPerFrame number of bits per audio frame.
        @param nchannels number of audio channels.
        @param signed true if signed format, false otherwise.
     */
    public RTAudioIn(float srate,int bitsPerFrame,int nchannels,boolean signed) {
        initAudio(0,srate,bitsPerFrame,nchannels,signed);
    }

    /** Read audio buffer from input queue and block if queue is empty.
        @param y buffer to write to.
        @param nsamples number of samples required.
     */
    public void read(float [] y,int nsamples) {
        if(2*nsamples > scratchBufsz) {
            scratchBufsz = 2 * nsamples;
            bbuf = new byte[scratchBufsz];
        }
        targetDataLine.read(bbuf,0,2*nsamples);
        FormatUtils.byteToFloat(y,bbuf,nsamples);
    }
    
}
